/// <reference path="../libs/lodash.d.ts" />
declare module chevrotain.lang {
    function classNameFromInstance(instance: any): string;
    function functionName(func: Function): string;
    /**
     * simple Hashtable between a string and some generic value
     * this should be removed once typescript supports ES6 style Hashtable
     */
    class HashTable<V> {
        private _state;
        keys(): string[];
        values(): V[];
        put(key: string, value: V): void;
        putAll(other: HashTable<V>): void;
        get(key: string): V;
        containsKey(key: string): boolean;
    }
}
declare module chevrotain.tokens {
    /**
     * utility to help the poor souls who are still stuck writing pure javascript 5.1
     * extend and create Token subclasses in a less verbose manner
     *
     * @param {string} tokenName the name of the new TokenClass
     * @param {*} patternOrParent Pa
     * @param {Function} parentConstructor the Token class to be extended
     * @returns {Function} a constructor for the new extended Token subclass
     */
    function extendToken(tokenName: string, patternOrParent?: any, parentConstructor?: Function): any;
    function tokenName(clazz: Function): string;
    class Token {
        image: string;
        offset: number;
        startLine: number;
        startColumn: number;
        endLine: number;
        endColumn: number;
        isInsertedInRecovery: boolean;
        /**
         * @param {string} image the textual representation of the Token as it appeared in the text
         * @param {number} offset offset of the first character of the Token
         * @param {number} startLine line of the first character of the Token
         * @param {number} startColumn column of the first character of the Token
         * @param {number} endLine line of the last character of the Token
         * @param {number} endColumn column of the last character of the Token
         *
         * Things to note:
         * * "do"  {startColumn : 1, endColumn: 2} --> the range is inclusive to exclusive 1...2 (2 chars long).
         * * "\n"  {startLine : 1, endLine: 1} --> a lineTerminator as the last character does not effect the Token's line numbering.
         * * "'hello\tworld\uBBBB'"  {image: "'hello\tworld\uBBBB'"} --> a Token's image is the "literal" text
         *                                                              (unicode escaping is untouched).
         */
        constructor(image: string, offset: number, startLine: number, startColumn: number, endLine?: number, endColumn?: number);
    }
    type VirtualTokenClass = Function;
    class VirtualToken extends Token {
        constructor();
    }
}
declare module chevrotain.lexer {
    import tok = chevrotain.tokens;
    interface ILexingResult {
        tokens: tok.Token[];
        groups: {
            [groupName: string]: tok.Token;
        };
        errors: ILexingError[];
    }
    interface ILexingError {
        line: number;
        column: number;
        message: string;
    }
    type TokenConstructor = Function;
    class Lexer {
        protected tokenClasses: TokenConstructor[];
        static SKIPPED: {
            description: string;
        };
        static NA: RegExp;
        protected allPatterns: RegExp[];
        protected patternIdxToClass: Function[];
        protected patternIdxToGroup: boolean[];
        protected patternIdxToLongerAltIdx: number[];
        protected patternIdxToCanLineTerminator: boolean[];
        protected emptyGroups: {
            [groupName: string]: tok.Token;
        };
        /**
         * @param {Function[]} tokenClasses constructor functions for the Tokens types this scanner will support
         *                     These constructors must be in one of three forms:
         *
         *  1. With a PATTERN property that has a RegExp value for tokens to match:
         *     example: -->class Integer extends tok.Token { static PATTERN = /[1-9]\d }<--
         *
         *  2. With a PATTERN property that has a RegExp value AND an IGNORE property with boolean value true.
         *     These tokens will be matched but not as part of the main token vector.
         *     this is usually used for ignoring whitespace/comments
         *     example: -->    class Whitespace extends tok.Token { static PATTERN = /(\t| )/; static IGNORE = true}<--
         *
         *  3. With a PATTERN property that has the value of the var NA define in this module.
         *     This is a convenience form used to avoid matching Token classes that only act as categories.
         *     example: -->class Keyword extends tok.Token { static PATTERN = NA }<--
         *
         *
         *   The following RegExp patterns are not supported:
         *   a. '$' for match at end of input
         *   b. /b global flag
         *   c. /m multi-line flag
         *
         *   The Lexer will identify the first pattern the matches, Therefor the order of Token Constructors passed
         *   To the SimpleLexer's constructor is meaningful. If two patterns may match the same string, the longer one
         *   should be before the shorter one.
         *
         *   Note that there are situations in which we may wish to place the longer pattern after the shorter one.
         *   For example: keywords vs Identifiers.
         *   'do'(/do/) and 'done'(/w+)
         *
         *   * If the Identifier pattern appears before the 'do' pattern both 'do' and 'done'
         *     will be lexed as an Identifier.
         *
         *   * If the 'do' pattern appears before the Identifier pattern 'do' will be lexed correctly as a keyword.
         *     however 'done' will be lexed as TWO tokens keyword 'do' and identifier 'ne'.
         *
         *   To resolve this problem, add a static property on the keyword's Tokens constructor named: LONGER_ALT
         *   example:
         *
         *       export class Identifier extends Keyword { static PATTERN = /[_a-zA-Z][_a-zA-Z0-9]/ }
         *       export class Keyword extends tok.Token {
         *          static PATTERN = lex.NA
         *          static LONGER_ALT = Identifier
         *       }
         *       export class Do extends Keyword { static PATTERN = /do/ }
         *       export class While extends Keyword { static PATTERN = /while/ }
         *       export class Return extends Keyword { static PATTERN = /return/ }
         *
         *   The lexer will then also attempt to match a (longer) Identifier each time a keyword is matched
         *
         *
         */
        constructor(tokenClasses: TokenConstructor[]);
        /**
         * Will lex(Tokenize) a string.
         * Note that this can be called repeatedly on different strings as this method
         * does not modify the state of the Lexer.
         *
         * @param {string} text the string to lex
         * @returns {{tokens: {Token}[], errors: string[]}}
         */
        tokenize(text: string): ILexingResult;
    }
    interface IAnalyzeResult {
        allPatterns: RegExp[];
        patternIdxToClass: Function[];
        patternIdxToGroup: any[];
        patternIdxToLongerAltIdx: number[];
        patternIdxToCanLineTerminator: boolean[];
        emptyGroups: {
            [groupName: string]: tok.Token;
        };
    }
    function analyzeTokenClasses(tokenClasses: TokenConstructor[]): IAnalyzeResult;
    function validatePatterns(tokenClasses: TokenConstructor[]): void;
    function findMissingPatterns(tokenClasses: TokenConstructor[]): string[];
    function findInvalidPatterns(tokenClasses: TokenConstructor[]): string[];
    function findEndOfInputAnchor(tokenClasses: TokenConstructor[]): string[];
    function findUnsupportedFlags(tokenClasses: TokenConstructor[]): string[];
    function findDuplicatePatterns(tokenClasses: TokenConstructor[]): string[];
    function findInvalidGroupType(tokenClasses: TokenConstructor[]): string[];
    function addStartOfInput(pattern: RegExp): RegExp;
    function countLineTerminators(text: string): number;
}
declare module chevrotain.tree {
    import tok = chevrotain.tokens;
    class ParseTree {
        payload: tok.Token;
        children: ParseTree[];
        getImage(): string;
        getLine(): number;
        getColumn(): number;
        constructor(payload: tok.Token, children?: ParseTree[]);
    }
    /**
     * convenience factory for ParseTrees
     *
     * @param {Function|Token} tokenOrTokenClass The Token instance to be used as the root node, or a constructor Function
     *                         that will create the root node.
     * @param {ParseTree[]} children The sub nodes of the ParseTree to the built
     * @returns {ParseTree}
     */
    function PT(tokenOrTokenClass: Function | tok.Token, children?: ParseTree[]): ParseTree;
}
declare module chevrotain.range {
    interface IRange {
        start: number;
        end: number;
        contains(num: number): boolean;
        containsRange(other: IRange): boolean;
        isContainedInRange(other: IRange): boolean;
        strictlyContainsRange(other: IRange): boolean;
        isStrictlyContainedInRange(other: IRange): boolean;
    }
    class Range implements IRange {
        start: number;
        end: number;
        constructor(start: number, end: number);
        contains(num: number): boolean;
        containsRange(other: IRange): boolean;
        isContainedInRange(other: IRange): boolean;
        strictlyContainsRange(other: IRange): boolean;
        isStrictlyContainedInRange(other: IRange): boolean;
    }
    function isValidRange(start: number, end: number): boolean;
}
declare module chevrotain.constants {
    var IN: string;
}
declare module chevrotain.path {
    /**
     * this interfaces defines the path the parser "took" to reach a certain position
     * in the grammar.
     */
    interface IGrammarPath {
        ruleStack: string[];
        occurrenceStack: number[];
    }
    interface ITokenGrammarPath extends IGrammarPath {
        lastTok: Function;
        lastTokOccurrence: number;
    }
    interface IRuleGrammarPath extends IGrammarPath {
        occurrence: number;
    }
}
declare module chevrotain.gast {
    interface IProduction {
        accept(visitor: GAstVisitor): void;
        dslName: string;
    }
    interface IProductionWithOccurrence extends IProduction {
        occurrenceInParent: number;
        implicitOccurrenceIndex: boolean;
    }
    class AbstractProduction implements IProduction {
        definition: IProduction[];
        implicitOccurrenceIndex: boolean;
        dslName: string;
        constructor(definition: IProduction[]);
        accept(visitor: GAstVisitor): void;
    }
    class ProdRef extends AbstractProduction implements IProductionWithOccurrence {
        refProdName: string;
        ref: TOP_LEVEL;
        occurrenceInParent: number;
        dslName: string;
        constructor(refProdName: string, ref?: TOP_LEVEL, occurrenceInParent?: number);
        definition: IProduction[];
        accept(visitor: GAstVisitor): void;
    }
    class TOP_LEVEL extends AbstractProduction {
        name: string;
        constructor(name: string, definition: IProduction[]);
    }
    class FLAT extends AbstractProduction {
        constructor(definition: IProduction[]);
    }
    class OPTION extends AbstractProduction implements IProductionWithOccurrence {
        occurrenceInParent: number;
        constructor(definition: IProduction[], occurrenceInParent?: number);
    }
    class AT_LEAST_ONE extends AbstractProduction implements IProductionWithOccurrence {
        occurrenceInParent: number;
        constructor(definition: IProduction[], occurrenceInParent?: number);
    }
    class MANY extends AbstractProduction implements IProductionWithOccurrence {
        occurrenceInParent: number;
        constructor(definition: IProduction[], occurrenceInParent?: number);
    }
    class OR extends AbstractProduction implements IProductionWithOccurrence {
        occurrenceInParent: number;
        constructor(definition: IProduction[], occurrenceInParent?: number);
    }
    class Terminal implements IProductionWithOccurrence {
        terminalType: Function;
        occurrenceInParent: number;
        dslName: string;
        implicitOccurrenceIndex: boolean;
        constructor(terminalType: Function, occurrenceInParent?: number);
        accept(visitor: GAstVisitor): void;
    }
    function isSequenceProd(prod: IProduction): boolean;
    function isOptionalProd(prod: IProduction): boolean;
    function isBranchingProd(prod: IProduction): boolean;
    class GAstVisitor {
        visit(node: IProduction): void;
        visitProdRef(node: ProdRef): void;
        visitFLAT(node: FLAT): void;
        visitOPTION(node: OPTION): void;
        visitAT_LEAST_ONE(node: AT_LEAST_ONE): void;
        visitMANY(node: MANY): void;
        visitOR(node: OR): void;
        visitTerminal(node: Terminal): void;
    }
}
declare module chevrotain.first {
    import gast = chevrotain.gast;
    function first(prod: gast.IProduction): Function[];
    function firstForSequence(prod: gast.AbstractProduction): Function[];
    function firstForBranching(prod: gast.AbstractProduction): Function[];
    function firstForTerminal(terminal: gast.Terminal): Function[];
}
declare module chevrotain.rest {
    import g = chevrotain.gast;
    class RestWalker {
        walk(prod: g.AbstractProduction, prevRest?: any[]): void;
        walkTerminal(terminal: g.Terminal, currRest: g.IProduction[], prevRest: g.IProduction[]): void;
        walkProdRef(refProd: g.ProdRef, currRest: g.IProduction[], prevRest: g.IProduction[]): void;
        walkFlat(flatProd: g.FLAT, currRest: g.IProduction[], prevRest: g.IProduction[]): void;
        walkOption(optionProd: g.OPTION, currRest: g.IProduction[], prevRest: g.IProduction[]): void;
        walkAtLeastOne(atLeastOneProd: g.AT_LEAST_ONE, currRest: g.IProduction[], prevRest: g.IProduction[]): void;
        walkMany(manyProd: g.MANY, currRest: g.IProduction[], prevRest: g.IProduction[]): void;
        walkOr(orProd: g.OR, currRest: g.IProduction[], prevRest: g.IProduction[]): void;
    }
}
declare module chevrotain.follow {
    import g = chevrotain.gast;
    import r = chevrotain.rest;
    import lang = chevrotain.lang;
    class ResyncFollowsWalker extends r.RestWalker {
        private topProd;
        follows: lang.HashTable<Function[]>;
        constructor(topProd: g.TOP_LEVEL);
        startWalking(): lang.HashTable<Function[]>;
        walkTerminal(terminal: g.Terminal, currRest: g.IProduction[], prevRest: g.IProduction[]): void;
        walkProdRef(refProd: g.ProdRef, currRest: g.IProduction[], prevRest: g.IProduction[]): void;
    }
    function computeAllProdsFollows(topProductions: g.TOP_LEVEL[]): lang.HashTable<Function[]>;
    function buildBetweenProdsFollowPrefix(inner: g.TOP_LEVEL, occurenceInParent: number): string;
    function buildInProdFollowPrefix(terminal: g.Terminal): string;
}
declare module chevrotain.interpreter {
    import g = chevrotain.gast;
    import r = chevrotain.rest;
    import p = chevrotain.path;
    class AbstractNextPossibleTokensWalker extends r.RestWalker {
        protected topProd: g.TOP_LEVEL;
        protected path: p.IGrammarPath;
        protected possibleTokTypes: Function[];
        protected ruleStack: string[];
        protected occurrenceStack: number[];
        protected nextProductionName: string;
        protected nextProductionOccurrence: number;
        protected found: boolean;
        protected isAtEndOfPath: boolean;
        constructor(topProd: g.TOP_LEVEL, path: p.IGrammarPath);
        startWalking(): Function[];
        walk(prod: g.AbstractProduction, prevRest?: gast.IProduction[]): void;
        walkProdRef(refProd: g.ProdRef, currRest: g.IProduction[], prevRest: g.IProduction[]): void;
        updateExpectedNext(): void;
    }
    class NextAfterTokenWalker extends AbstractNextPossibleTokensWalker {
        protected path: p.ITokenGrammarPath;
        private nextTerminalName;
        private nextTerminalOccurrence;
        constructor(topProd: g.TOP_LEVEL, path: p.ITokenGrammarPath);
        walkTerminal(terminal: g.Terminal, currRest: g.IProduction[], prevRest: g.IProduction[]): void;
    }
    class NextInsideOptionWalker extends AbstractNextPossibleTokensWalker {
        protected path: p.IRuleGrammarPath;
        private nextOptionOccurrence;
        constructor(topProd: g.TOP_LEVEL, path: p.IRuleGrammarPath);
        walkOption(optionProd: g.OPTION, currRest: g.IProduction[], prevRest: g.IProduction[]): void;
    }
    class NextInsideManyWalker extends AbstractNextPossibleTokensWalker {
        protected path: p.IRuleGrammarPath;
        private nextOccurrence;
        constructor(topProd: g.TOP_LEVEL, path: p.IRuleGrammarPath);
        walkMany(manyProd: g.MANY, currRest: g.IProduction[], prevRest: g.IProduction[]): void;
    }
    class NextInsideAtLeastOneWalker extends AbstractNextPossibleTokensWalker {
        protected path: p.IRuleGrammarPath;
        private nextOccurrence;
        constructor(topProd: g.TOP_LEVEL, path: p.IRuleGrammarPath);
        walkAtLeastOne(atLeastOneProd: g.AT_LEAST_ONE, currRest: g.IProduction[], prevRest: g.IProduction[]): void;
    }
    type AlternativesFirstTokens = Function[][];
    class NextInsideOrWalker extends r.RestWalker {
        protected topRule: g.TOP_LEVEL;
        protected occurrence: number;
        result: AlternativesFirstTokens;
        constructor(topRule: g.TOP_LEVEL, occurrence: number);
        startWalking(): AlternativesFirstTokens;
        walkOr(orProd: g.OR, currRest: g.IProduction[], prevRest: g.IProduction[]): void;
    }
    interface IFirstAfterRepetition {
        token: Function;
        occurrence: number;
        isEndOfRule: boolean;
    }
    /**
     * This walker only "walks" a single "TOP" level in the Grammar Ast, this means
     * it never "follows" production refs
     */
    class AbstractNextTerminalAfterProductionWalker extends r.RestWalker {
        protected topRule: g.TOP_LEVEL;
        protected occurrence: number;
        protected result: {
            token: any;
            occurrence: any;
            isEndOfRule: any;
        };
        constructor(topRule: g.TOP_LEVEL, occurrence: number);
        startWalking(): IFirstAfterRepetition;
    }
    class NextTerminalAfterManyWalker extends AbstractNextTerminalAfterProductionWalker {
        walkMany(manyProd: g.MANY, currRest: g.IProduction[], prevRest: g.IProduction[]): void;
    }
    class NextTerminalAfterAtLeastOneWalker extends AbstractNextTerminalAfterProductionWalker {
        walkAtLeastOne(atLeastOneProd: g.AT_LEAST_ONE, currRest: g.IProduction[], prevRest: g.IProduction[]): void;
    }
}
/**
 *  Module used to cache static information about parsers,
 */
declare module chevrotain.cache {
    var CLASS_TO_VALIDTATION_ERRORS: lang.HashTable<string[]>;
    var CLASS_TO_SELF_ANALYSIS_DONE: lang.HashTable<boolean>;
    var CLASS_TO_GRAMMAR_PRODUCTIONS: lang.HashTable<lang.HashTable<gast.TOP_LEVEL>>;
    function getProductionsForClass(className: string): lang.HashTable<gast.TOP_LEVEL>;
    var CLASS_TO_RESYNC_FOLLOW_SETS: lang.HashTable<lang.HashTable<Function[]>>;
    function getResyncFollowsForClass(className: string): lang.HashTable<Function[]>;
    function setResyncFollowsForClass(className: string, followSet: lang.HashTable<Function[]>): void;
    var CLASS_TO_LOOKAHEAD_FUNCS: lang.HashTable<lang.HashTable<Function>>;
    function getLookaheadFuncsForClass(className: string): lang.HashTable<Function>;
    var CLASS_TO_FIRST_AFTER_REPETITION: lang.HashTable<lang.HashTable<interpreter.IFirstAfterRepetition>>;
    function getFirstAfterRepForClass(className: string): lang.HashTable<interpreter.IFirstAfterRepetition>;
    var CLASS_TO_OR_LA_CACHE: lang.HashTable<lang.HashTable<string>[]>;
    var CLASS_TO_MANY_LA_CACHE: lang.HashTable<lang.HashTable<string>[]>;
    var CLASS_TO_AT_LEAST_ONE_LA_CACHE: lang.HashTable<lang.HashTable<string>[]>;
    var CLASS_TO_OPTION_LA_CACHE: lang.HashTable<lang.HashTable<string>[]>;
    var MAX_OCCURRENCE_INDEX: number;
    function initLookAheadKeyCache(className: any): void;
}
declare module chevrotain.lookahead {
    import gast = chevrotain.gast;
    function buildLookaheadForTopLevel(rule: gast.TOP_LEVEL): () => boolean;
    function buildLookaheadForOption(optionOccurrence: number, ruleGrammar: gast.TOP_LEVEL): () => boolean;
    function buildLookaheadForMany(manyOccurrence: number, ruleGrammar: gast.TOP_LEVEL): () => boolean;
    function buildLookaheadForAtLeastOne(manyOccurrence: number, ruleGrammar: gast.TOP_LEVEL): () => boolean;
    function buildLookaheadForOr(orOccurrence: number, ruleGrammar: gast.TOP_LEVEL, ignoreAmbiguities?: boolean): () => number;
    interface IAmbiguityDescriptor {
        alts: number[];
        token: Function;
    }
    function checkAlternativesAmbiguities(alternativesTokens: Function[][]): IAmbiguityDescriptor[];
}
declare module chevrotain.gastBuilder {
    import r = chevrotain.range;
    import gast = chevrotain.gast;
    import lang = chevrotain.lang;
    enum ProdType {
        OPTION = 0,
        OR = 1,
        MANY = 2,
        AT_LEAST_ONE = 3,
        REF = 4,
        TERMINAL = 5,
        FLAT = 6,
    }
    interface IProdRange {
        range: r.IRange;
        text: string;
        type: ProdType;
    }
    interface ITerminalNameToConstructor {
        [fqn: string]: Function;
    }
    var terminalNameToConstructor: ITerminalNameToConstructor;
    function buildTopProduction(impelText: string, name: string, terminals: ITerminalNameToConstructor): gast.TOP_LEVEL;
    function buildProdGast(prodRange: IProdRange, allRanges: IProdRange[]): gast.IProduction;
    function getDirectlyContainedRanges(y: r.IRange, prodRanges: IProdRange[]): IProdRange[];
    function removeComments(text: string): string;
    function createRanges(text: string): IProdRange[];
    function createTerminalRanges(text: string): IProdRange[];
    function createRefsRanges(text: string): IProdRange[];
    function createAtLeastOneRanges(text: string): IProdRange[];
    function createManyRanges(text: string): IProdRange[];
    function createOptionRanges(text: string): IProdRange[];
    function createOrRanges(text: any): IProdRange[];
    function createOrPartRanges(orRanges: IProdRange[]): IProdRange[];
    function findClosingOffset(opening: string, closing: string, start: number, text: string): number;
    class GastRefResolverVisitor extends gast.GAstVisitor {
        private nameToProd;
        constructor(nameToProd: lang.HashTable<gast.TOP_LEVEL>);
        resolveRefs(): void;
        visitProdRef(node: gast.ProdRef): void;
    }
}
declare module chevrotain.validations {
    import gast = chevrotain.gast;
    function validateGrammar(topLevels: gast.TOP_LEVEL[]): string[];
    function identifyProductionForDuplicates(prod: gast.IProductionWithOccurrence): string;
    class OccurrenceValidationCollector extends gast.GAstVisitor {
        allProductions: gast.IProduction[];
        visitProdRef(subrule: gast.ProdRef): void;
        visitOPTION(option: gast.OPTION): void;
        visitAT_LEAST_ONE(atLeastOne: gast.AT_LEAST_ONE): void;
        visitMANY(many: gast.MANY): void;
        visitOR(or: gast.OR): void;
        visitTerminal(terminal: gast.Terminal): void;
    }
}
declare module chevrotain.recognizer {
    import tok = chevrotain.tokens;
    import gast = chevrotain.gast;
    import interp = chevrotain.interpreter;
    import lang = chevrotain.lang;
    import gastBuilder = chevrotain.gastBuilder;
    function isRecognitionException(error: Error): boolean;
    function MismatchedTokenException(message: string, token: tok.Token): void;
    function NoViableAltException(message: string, token: tok.Token): void;
    function NotAllInputParsedException(message: string, token: tok.Token): void;
    function EarlyExitException(message: string, token: tok.Token): void;
    class EOF extends tok.VirtualToken {
    }
    /**
     * OR([
     *  { WHEN:LA1, THEN_DO:XXX },
     *  { WHEN:LA2, THEN_DO:YYY },
     *  { WHEN:LA3, THEN_DO:ZZZ },
     * ])
     */
    interface IOrAlt<T> {
        WHEN: () => boolean;
        THEN_DO: () => T;
    }
    /**
     * OR([
     *  {ALT:XXX },
     *  {ALT:YYY },
     *  {ALT:ZZZ }
     * ])
     */
    interface IOrAltImplicit<T> {
        ALT: () => T;
    }
    interface IBaseRecogState {
        errors: Error[];
        inputIdx: number;
    }
    interface IErrorRecoveryRecogState extends IBaseRecogState {
        RULE_STACK: string[];
    }
    type LookAheadFunc = () => boolean;
    type GrammarAction = () => void;
    var IGNORE_AMBIGUITIES: boolean;
    var NO_RESYNC: boolean;
    /**
     * This is The BaseRecognizer, this should generally not be extended directly, instead
     * the BaseIntrospectionRecognizer should be used as the base class for external users.
     * as it has the full feature set.
     */
    class BaseRecognizer {
        errors: Error[];
        protected _input: tok.Token[];
        protected inputIdx: number;
        protected isBackTrackingStack: any[];
        protected className: string;
        constructor(input?: tok.Token[]);
        input: tok.Token[];
        protected isBackTracking(): boolean;
        reset(): void;
        isAtEndOfInput(): boolean;
        protected SAVE_ERROR(error: Error): Error;
        protected NEXT_TOKEN(): tok.Token;
        protected SKIP_TOKEN(): tok.Token;
        protected CONSUME(tokClass: Function): tok.Token;
        protected LA(howMuch: number): tok.Token;
        /**
         *
         * @param grammarRule the rule to try and parse in backtracking mode
         * @param isValid a predicate that given the result of the parse attempt will "decide" if the parse was successfully or not
         * @return a lookahead function that will try to parse the given grammarRule and will return true if succeed
         */
        protected BACKTRACK<T>(grammarRule: (...args) => T, isValid: (T) => boolean): () => boolean;
        protected saveRecogState(): IBaseRecogState;
        protected reloadRecogState(newState: IBaseRecogState): void;
        protected OPTION(condition: LookAheadFunc, action: GrammarAction): boolean;
        protected OR<T>(alts: IOrAlt<T>[], errMsgTypes: string): T;
        protected MANY(lookAheadFunc: LookAheadFunc, action: GrammarAction): void;
        protected AT_LEAST_ONE(lookAheadFunc: LookAheadFunc, action: GrammarAction, errMsg: string): void;
        protected raiseNoAltException(errMsgTypes: string): void;
    }
    function InRuleRecoveryException(message: string): void;
    interface IFollowKey {
        ruleName: string;
        idxInCallingRule: number;
        inRule: string;
    }
    /**
     * A Recognizer capable of self analysis to determine it's grammar structure
     * This is used for more advanced features requiring such information.
     * for example: Error Recovery, Automatic lookahead calculation
     */
    class Parser extends BaseRecognizer {
        protected static performSelfAnalysis(classInstance: Parser): void;
        protected RULE_STACK: string[];
        protected RULE_OCCURRENCE_STACK: number[];
        protected tokensMap: gastBuilder.ITerminalNameToConstructor;
        protected firstAfterRepMap: lang.HashTable<interp.IFirstAfterRepetition>;
        protected classLAFuncs: lang.HashTable<Function>;
        protected orLookaheadKeys: lang.HashTable<string>[];
        protected manyLookaheadKeys: lang.HashTable<string>[];
        protected atLeastOneLookaheadKeys: lang.HashTable<string>[];
        protected optionLookaheadKeys: lang.HashTable<string>[];
        constructor(input: tok.Token[], tokensMapOrArr: gastBuilder.ITerminalNameToConstructor | Function[]);
        reset(): void;
        /**
         * Convenience method equivalent to CONSUME1
         * @see CONSUME1
         */
        protected CONSUME(tokClass: Function): tok.Token;
        /**
         *
         * A Parsing DSL method use to consume a single terminal Token.
         * a Token will be consumed, IFF the next token in the token vector is an instanceof tokClass.
         * otherwise the parser will attempt to perform error recovery.
         *
         * The index in the method name indicates the unique occurrence of a terminal consumption
         * inside a the top level rule. What this means is that if a terminal appears
         * more than once in a single rule, each appearance must have a difference index.
         *
         * for example:
         *
         * function parseQualifiedName() {
         *    this.CONSUME1(Identifier);
         *    this.MANY(()=> {
         *       this.CONSUME1(Dot);
         *       this.CONSUME2(Identifier); // <-- here we use CONSUME2 because the terminal
         *    });                           //     'Identifier' has already appeared previously in the
         *                                  //     the rule 'parseQualifiedName'
         * }
         *
         * @param {Function} tokClass A constructor function specifying the type of token
         *        to be consumed.
         *
         * @returns {chevrotain.tokens.Token} The consumed token.
         */
        protected CONSUME1(tokClass: Function): tok.Token;
        /**
         * @see CONSUME1
         */
        protected CONSUME2(tokClass: Function): tok.Token;
        /**
         * @see CONSUME1
         */
        protected CONSUME3(tokClass: Function): tok.Token;
        /**
         * @see CONSUME1
         */
        protected CONSUME4(tokClass: Function): tok.Token;
        /**
         * @see CONSUME1
         */
        protected CONSUME5(tokClass: Function): tok.Token;
        /**
         * Convenience method equivalent to SUBRULE1
         * @see SUBRULE1
         */
        protected SUBRULE<T>(ruleToCall: (number) => T, args?: any[]): T;
        /**
         * The Parsing DSL Method is used by one rule to call another.
         *
         * This may seem redundant as it does not actually do much.
         * However using it is mandatory for all sub rule invocations.
         * calling another rule without wrapping in SUBRULE(...)
         * will cause errors/mistakes in the Recognizer's self analysis
         * which will lead to errors in error recovery/automatic lookahead calcualtion
         * and any other functionality relying on the Recognizer's self analysis
         * output.
         *
         * As in CONSUME the index in the method name indicates the occurrence
         * of the sub rule invocation in its rule.
         *
         * @param {Function} ruleToCall the rule to invoke
         * @param {*[]} args the arguments to pass to the invoked subrule
         * @returns {*} the result of invoking ruleToCall
         */
        protected SUBRULE1<T>(ruleToCall: (number) => T, args?: any[]): T;
        /**
         * @see SUBRULE1
         */
        protected SUBRULE2<T>(ruleToCall: (number) => T, args?: any[]): T;
        /**
         * @see SUBRULE1
         */
        protected SUBRULE3<T>(ruleToCall: (number) => T, args?: any[]): T;
        /**
         * @see SUBRULE1
         */
        protected SUBRULE4<T>(ruleToCall: (number) => T, args?: any[]): T;
        /**
         * @see SUBRULE1
         */
        protected SUBRULE5<T>(ruleToCall: (number) => T, args?: any[]): T;
        /**
         * Convenience method equivalent to OPTION1
         * @see OPTION1
         */
        protected OPTION(laFuncOrAction: LookAheadFunc | GrammarAction, action?: GrammarAction): boolean;
        /**
         * Parsing DSL Method that Indicates an Optional production
         * in EBNF notation: [...]
         *
         * note that the 'action' param is optional. so both of the following forms are valid:
         *
         * short: this.OPTION(()=>{ this.CONSUME(Digit});
         * long: this.OPTION(isDigit, ()=>{ this.CONSUME(Digit});
         *
         * using the short form is recommended as it will compute the lookahead function
         * automatically. however this currently has one limitation:
         * It only works if the lookahead for the grammar is one.
         *
         * As in CONSUME the index in the method name indicates the occurrence
         * of the optional production in it's top rule.
         *
         * @param {Function} laFuncOrAction The lookahead function that 'decides'
         *                                  whether or not the OPTION's action will be
         *                                  invoked or the action to optionally invoke
         * @param {Function} [action] The action to optionally invoke.
         *
         * @returns {boolean} true iff the OPTION's action has been invoked
         */
        protected OPTION1(laFuncOrAction: LookAheadFunc | GrammarAction, action?: GrammarAction): boolean;
        /**
         * @see OPTION1
         */
        protected OPTION2(laFuncOrAction: LookAheadFunc | GrammarAction, action?: GrammarAction): boolean;
        /**
         * @see OPTION1
         */
        protected OPTION3(laFuncOrAction: LookAheadFunc | GrammarAction, action?: GrammarAction): boolean;
        /**
         * @see OPTION1
         */
        protected OPTION4(laFuncOrAction: LookAheadFunc | GrammarAction, action?: GrammarAction): boolean;
        /**
         * @see OPTION1
         */
        protected OPTION5(laFuncOrAction: LookAheadFunc | GrammarAction, action?: GrammarAction): boolean;
        /**
         * Convenience method equivalent to OR1
         * @see OR1
         */
        protected OR<T>(alts: IOrAlt<T>[] | IOrAltImplicit<T>[], errMsgTypes: string, ignoreAmbiguities?: boolean): T;
        /**
         * Parsing DSL method that indicates a choice between a set of alternatives must be made.
         * This is equivalent to EBNF alternation (A | B | C | D ...)
         *
         * There are two forms:
         *
         * short: this.OR([
         *           {ALT:()=>{this.CONSUME(One)}},
         *           {ALT:()=>{this.CONSUME(Two)}},
         *           {ALT:()=>{this.CONSUME(Three)}},
         *        ], "a number")
         *
         * long: this.OR([
         *           {WHEN: isOne, THEN_DO:()=>{this.CONSUME(One)}},
         *           {WHEN: isTwo, THEN_DO:()=>{this.CONSUME(Two)}},
         *           {WHEN: isThree, THEN_DO:()=>{this.CONSUME(Three)}},
         *        ], "a number")
         *
         * using the short form is recommended as it will compute the lookahead function
         * automatically. however this currently has one limitation:
         * It only works if the lookahead for the grammar is one.
         *
         * As in CONSUME the index in the method name indicates the occurrence
         * of the alternation production in it's top rule.
         *
         * @param {{ALT:Function}[] | {WHEN:Function, THEN_DO:Function}[]} alts An array of alternatives
         * @param {string} errMsgTypes A description for the alternatives used in error messages
         * @returns {*} The result of invoking the chosen alternative
         * @param {boolean} [ignoreAmbiguities] if true this will ignore ambiguities caused when two alternatives can not
         *                                      be distinguished by a lookahead of one. enabling this means the first alternative
         *                                      that matches will be taken. This is sometimes the grammar's intent.
         *                                      * only enable this if you know what you are doing!
         */
        protected OR1<T>(alts: IOrAlt<T>[] | IOrAltImplicit<T>[], errMsgTypes: string, ignoreAmbiguities?: boolean): T;
        /**
         * @see OR1
         */
        protected OR2<T>(alts: IOrAlt<T>[] | IOrAltImplicit<T>[], errMsgTypes: string, ignoreAmbiguities?: boolean): T;
        /**
         * @see OR1
         */
        protected OR3<T>(alts: IOrAlt<T>[] | IOrAltImplicit<T>[], errMsgTypes: string, ignoreAmbiguities?: boolean): T;
        /**
         * @see OR1
         */
        protected OR4<T>(alts: IOrAlt<T>[] | IOrAltImplicit<T>[], errMsgTypes: string, ignoreAmbiguities?: boolean): T;
        /**
         * @see OR1
         */
        protected OR5<T>(alts: IOrAlt<T>[] | IOrAltImplicit<T>[], errMsgTypes: string, ignoreAmbiguities?: boolean): T;
        /**
         * Convenience method equivalent to MANY1
         * @see MANY1
         */
        protected MANY(lookAheadFunc: LookAheadFunc | GrammarAction, action?: GrammarAction): void;
        /**
         * Parsing DSL method, that indicates a repetition of zero or more.
         * This is equivalent to EBNF repetition {...}
         *
         * note that the 'action' param is optional. so both of the following forms are valid:
         *
         * short: this.MANY(()=>{
         *                       this.CONSUME(Comma};
         *                       this.CONSUME(Digit});
         * long: this.MANY(isComma, ()=>{
         *                       this.CONSUME(Comma};
         *                       this.CONSUME(Digit});
         *
         * using the short form is recommended as it will compute the lookahead function
         * automatically. however this currently has one limitation:
         * It only works if the lookahead for the grammar is one.
         *
         * As in CONSUME the index in the method name indicates the occurrence
         * of the repetition production in it's top rule.
         *
         * @param {Function} laFuncOrAction The lookahead function that 'decides'
         *                                  whether or not the MANY's action will be
         *                                  invoked or the action to optionally invoke
         * @param {Function} [action] The action to optionally invoke.
         */
        protected MANY1(laFuncOrAction: LookAheadFunc | GrammarAction, action?: GrammarAction): void;
        /**
         * @see MANY1
         */
        protected MANY2(laFuncOrAction: LookAheadFunc | GrammarAction, action?: GrammarAction): void;
        /**
         * @see MANY1
         */
        protected MANY3(laFuncOrAction: LookAheadFunc | GrammarAction, action?: GrammarAction): void;
        /**
         * @see MANY1
         */
        protected MANY4(laFuncOrAction: LookAheadFunc | GrammarAction, action?: GrammarAction): void;
        /**
         * @see MANY1
         */
        protected MANY5(laFuncOrAction: LookAheadFunc | GrammarAction, action?: GrammarAction): void;
        /**
         * Convenience method equivalent to AT_LEAST_ONE1
         * @see AT_LEAST_ONE1
         */
        protected AT_LEAST_ONE(laFuncOrAction: LookAheadFunc | GrammarAction, action: GrammarAction | string, errMsg?: string): void;
        /**
         *
         * convenience method, same as MANY but the repetition is of one or more.
         * failing to match at least one repetition will result in a parsing error and
         * cause the parser to attempt error recovery.
         *
         * @see MANY1
         *
         * @param {Function} laFuncOrAction The lookahead function that 'decides'
         *                                  whether or not the AT_LEAST_ONE's action will be
         *                                  invoked or the action to optionally invoke
         * @param {Function} [action] The action to optionally invoke.
         * @param {string} [errMsg] short title/classification to what is being matched
         */
        protected AT_LEAST_ONE1(laFuncOrAction: LookAheadFunc | GrammarAction, action: GrammarAction | string, errMsg?: string): void;
        /**
         * @see AT_LEAST_ONE1
         */
        protected AT_LEAST_ONE2(laFuncOrAction: LookAheadFunc | GrammarAction, action: GrammarAction | string, errMsg?: string): void;
        /**
         * @see AT_LEAST_ONE1
         */
        protected AT_LEAST_ONE3(laFuncOrAction: LookAheadFunc | GrammarAction, action: GrammarAction | string, errMsg?: string): void;
        /**
         * @see AT_LEAST_ONE1
         */
        protected AT_LEAST_ONE4(laFuncOrAction: LookAheadFunc | GrammarAction, action: GrammarAction | string, errMsg?: string): void;
        /**
         * @see AT_LEAST_ONE1
         */
        protected AT_LEAST_ONE5(laFuncOrAction: LookAheadFunc | GrammarAction, action: GrammarAction | string, errMsg?: string): void;
        /**
         * Convenience method, same as RULE with doReSync=false
         * @see RULE
         */
        protected RULE_NO_RESYNC<T>(ruleName: string, impl: () => T, invalidRet: () => T): (idxInCallingRule: number, isEntryPoint?: boolean) => T;
        /**
         *
         * @param {string} ruleName The name of the Rule. must match the var it is assigned to.
         * @param {Function} impl The implementation of the Rule
         * @param {Function} [invalidRet] A function that will return the chosen invalid value for the rule in case of
         *                   re-sync recovery.
         * @param {boolean} [doReSync] enable or disable re-sync recovery for this rule. defaults to true
         * @returns {Function} The parsing rule which is the impl Function wrapped with the parsing logic that handles
         *                     Parser state / error recovery / ...
         */
        protected RULE<T>(ruleName: string, impl: (...implArgs: any[]) => T, invalidRet?: () => T, doReSync?: boolean): (idxInCallingRule?: number, ...args: any[]) => T;
        protected ruleInvocationStateUpdate(ruleName: string, idxInCallingRule: number): void;
        protected ruleFinallyStateUpdate(): void;
        private defaultInvalidReturn();
        protected ruleNamePattern: RegExp;
        protected definedRulesNames: string[];
        /**
         * @param ruleFuncName name of the Grammar rule
         * @throws Grammar validation errors if the name is invalid
         */
        protected validateRuleName(ruleFuncName: string): void;
        protected tryInRepetitionRecovery(grammarRule: Function, grammarRuleArgs: any[], lookAheadFunc: () => boolean, expectedTokType: Function): void;
        protected shouldInRepetitionRecoveryBeTried(expectTokAfterLastMatch?: Function, nextTokIdx?: number): boolean;
        protected getFollowsForInRuleRecovery(tokClass: Function, tokIdxInRule: any): Function[];
        protected getTokenToInsert(tokClass: Function): tok.Token;
        protected canTokenTypeBeInsertedInRecovery(tokClass: Function): boolean;
        protected tryInRuleRecovery(expectedTokType: Function, follows: Function[]): tok.Token;
        protected canPerformInRuleRecovery(expectedToken: Function, follows: Function[]): boolean;
        protected canRecoverWithSingleTokenInsertion(expectedTokType: Function, follows: Function[]): boolean;
        protected canRecoverWithSingleTokenDeletion(expectedTokType: Function): boolean;
        protected isInCurrentRuleReSyncSet(token: Function): boolean;
        protected findReSyncTokenType(): Function;
        protected getCurrFollowKey(): IFollowKey;
        protected buildFullFollowKeyStack(): IFollowKey[];
        protected flattenFollowSet(): Function[];
        protected getFollowSetFromFollowKey(followKey: IFollowKey): Function[];
        protected reSyncTo(tokClass: Function): void;
        private attemptInRepetitionRecovery(prodFunc, args, lookaheadFunc, prodName, prodOccurrence, nextToksWalker, prodKeys);
        private atLeastOneInternal(prodFunc, prodName, prodOccurrence, lookAheadFunc, action, errMsg?);
        private manyInternal(prodFunc, prodName, prodOccurrence, lookAheadFunc, action?);
        private orInternal<T>(alts, errMsgTypes, occurrence, ignoreAmbiguities);
        /**
         * @param tokClass The Type of Token we wish to consume (Reference to its constructor function)
         * @param idx occurrence index of consumed token in the invoking parser rule text
         *         for example:
         *         IDENT (DOT IDENT)*
         *         the first ident will have idx 1 and the second one idx 2
         *         * note that for the second ident the idx is always 2 even if its invoked 30 times in the same rule
         *           the idx is about the position in grammar (source code) and has nothing to do with a specific invocation
         *           details
         *
         * @returns the consumed Token
         */
        protected consumeInternal(tokClass: Function, idx: number): tok.Token;
        protected getKeyForAutomaticLookahead(prodName: string, prodKeys: lang.HashTable<string>[], occurrence: number): string;
        protected getLookaheadFuncForOption(occurence: number): () => boolean;
        protected getLookaheadFuncForOr(occurence: number, ignoreErrors: boolean): () => number;
        protected getLookaheadFuncForMany(occurence: number): () => boolean;
        protected getLookaheadFuncForAtLeastOne(occurence: number): () => boolean;
        protected isNextRule<T>(ruleName: string): boolean;
        protected getLookaheadFuncFor<T>(key: string, occurrence: number, laFuncBuilder: (number, any) => () => T, extraArgs?: any[]): () => T;
        protected saveRecogState(): IErrorRecoveryRecogState;
        protected reloadRecogState(newState: IErrorRecoveryRecogState): void;
        protected getGAstProductions(): lang.HashTable<gast.TOP_LEVEL>;
    }
}
/**
 * defines the public API of Chevrotain.
 * changes here may require major version change. (semVer)
 */
declare var CHEV_TEST_MODE: any;
declare var global: any;
declare var testMode: any;
declare var API: any;
